==> kbibtex-0.1.3.tar.bz2: kbibtex-0.1.3/src/bibtexfileexporterexternal.cpp <==
/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qprocess.h>
#include <qwaitcondition.h>
#include <qapplication.h>
#include <qbuffer.h>

#include <bibtexfileexporterbibtex.h>
#include <settings.h>
#include "bibtexfileexporterexternal.h"

namespace BibTeX
{

    BibTeXFileExporterExternal::BibTeXFileExporterExternal( Settings::Exporter exporter, Settings::FileFormat fileformat )
            : BibTeXFileExporter(), writeTo( NULL ), m_exporter( exporter ), m_fileformat( fileformat )
    {
        wc = new QWaitCondition();
    }

    BibTeXFileExporterExternal::~BibTeXFileExporterExternal()
    {
        delete wc;
    }

    bool BibTeXFileExporterExternal::save( QIODevice* iodevice, BibTeXFile* bibtexfile )
    {
        QBuffer buffer;
        buffer.open( IO_WriteOnly );
        BibTeXFileExporter * bibtexExporter = new BibTeXFileExporterBibTeX();
        bool result = bibtexExporter->save( &buffer, bibtexfile );
        buffer.close();
        delete bibtexExporter;

        if ( result )
            result = generateOutput( buffer, iodevice );

        return result;
    }

    bool BibTeXFileExporterExternal::save( QIODevice* iodevice, BibTeXElement* element )
    {
        QBuffer buffer;
        buffer.open( IO_WriteOnly );
        BibTeXFileExporter * bibtexExporter = new BibTeXFileExporterBibTeX();
        bool result = bibtexExporter->save( &buffer, element );
        buffer.close();
        delete bibtexExporter;

        if ( result )
            result = generateOutput( buffer, iodevice );

        return result;
    }

    bool BibTeXFileExporterExternal::generateOutput( QBuffer &input, QIODevice *output )
    {
        bool result = FALSE;

        QString commandLine;
        switch ( m_fileformat )
        {
        case Settings::formatHTML:
            switch ( m_exporter )
            {
            case Settings::exporterBibTeX2HTML:
                commandLine = "bibtex2html -s plain -a";
                break;
            case Settings::exporterBib2XHTML:
                commandLine = "bib2xhtml -s plain -u";
                break;
            case Settings::exporterBibConv:
                commandLine = "bibconv -informat=bibtex -outformat=html";
                break;
            default:
                return FALSE;
            }
            break;
        case Settings::formatEndNote:
            switch ( m_exporter )
            {
            case Settings::exporterBibConv:
                commandLine = "bibconv -informat=bibtex -outformat=endnote";
                break;
            default:
                return FALSE;
            }
            break;
        case Settings::formatProCite:
            switch ( m_exporter )
            {
            case Settings::exporterBibConv:
                commandLine = "bibconv -informat=bibtex -outformat=procite";
                break;
            default:
                return FALSE;
            }
            break;
        default:
            return FALSE;
        }
        QStringList args = QStringList::split( ' ', commandLine );

        writeTo = new QTextStream( output );

        QApplication::setOverrideCursor( Qt::waitCursor );

        process = new QProcess( args );
        connect( process, SIGNAL( processExited() ), this, SLOT( slotProcessExited() ) );
        connect( process, SIGNAL( readyReadStdout() ), this, SLOT( slotReadProcessOutput() ) );
        connect( process, SIGNAL( readyReadStderr() ), this, SLOT( slotReadProcessOutput() ) );
        connect( process, SIGNAL( wroteToStdin() ), this, SLOT( slotWroteToStdin() ) );

        if ( process->start() )
        {
            while ( !process->isRunning() )
            {
                wc->wait( 250 );
                qApp->processEvents();
            }
            qApp->processEvents();
            input.open( IO_ReadOnly );
            QByteArray buf = input.buffer();
            //             qDebug( "buf.size=%i", buf.size() );
            process->writeToStdin( buf );
            input.close();
            qApp->processEvents();
            while ( process->isRunning() )
            {
                wc->wait( 250 );
                qApp->processEvents();
            }

            result = process->normalExit();
        }

        disconnect( process, SIGNAL( wroteToStdin() ), this, SLOT( slotWroteToStdin() ) );
        disconnect( process, SIGNAL( readyReadStdout() ), this, SLOT( slotReadProcessOutput() ) );
        disconnect( process, SIGNAL( readyReadStderr() ), this, SLOT( slotReadProcessOutput() ) );
        disconnect( process, SIGNAL( processExited() ), this, SLOT( slotProcessExited() ) );
        delete( process );
        process = NULL;
        delete writeTo;
        writeTo = NULL;

        QApplication::restoreOverrideCursor();
        return result;
    }

    void BibTeXFileExporterExternal::slotProcessExited()
    {
        wc->wakeAll();
    }

    void BibTeXFileExporterExternal::slotReadProcessOutput()
    {
        if ( writeTo != NULL )
            while ( process->canReadLineStdout() )
            {
                QString line = process->readLineStdout();
                //                 qDebug( "line=%s", line.latin1() );
                ( *writeTo ) << line.latin1() << endl;
            }
        //         while ( process->canReadLineStderr() )
        //         {
        //             QString line = process->readLineStderr();
        //             qDebug( "stderr=%s", line.latin1() );
        //         }
    }

    void BibTeXFileExporterExternal::slotWroteToStdin()
    {
        process->closeStdin();
    }
}

#include "bibtexfileexporterexternal.moc"

==> kbibtex-0.1.4.tar.bz2: kbibtex-0.1.4/src/bibtexfileexporterexternal.cpp <==
/***************************************************************************
*   Copyright (C) 2004-2005 by Thomas Fischer                             *
*   fischer@unix-ag.uni-kl.de                                             *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/
#include <qprocess.h>
#include <qwaitcondition.h>
#include <qapplication.h>
#include <qbuffer.h>

#include <bibtexfileexporterbibtex.h>
#include <settings.h>
#include "bibtexfileexporterexternal.h"

namespace BibTeX
{

    BibTeXFileExporterExternal::BibTeXFileExporterExternal( KBibTeX::Settings::Exporter exporter, KBibTeX::Settings::FileFormat fileformat )
            : BibTeXFileExporter(), writeTo( NULL ), m_exporter( exporter ), m_fileformat( fileformat )
    {
        wc = new QWaitCondition();
    }

    BibTeXFileExporterExternal::~BibTeXFileExporterExternal()
    {
        delete wc;
    }

    bool BibTeXFileExporterExternal::save( QIODevice* iodevice, BibTeXFile* bibtexfile, QStringList *errorLog )
    {
        QBuffer buffer;
        buffer.open( IO_WriteOnly );
        BibTeXFileExporter * bibtexExporter = new BibTeXFileExporterBibTeX();
        bool result = bibtexExporter->save( &buffer, bibtexfile, errorLog );
        buffer.close();
        delete bibtexExporter;

        if ( result )
            result = generateOutput( buffer, iodevice );

        return result;
    }

    bool BibTeXFileExporterExternal::save( QIODevice* iodevice, BibTeXElement* element, QStringList *errorLog )
    {
        QBuffer buffer;
        buffer.open( IO_WriteOnly );
        BibTeXFileExporter * bibtexExporter = new BibTeXFileExporterBibTeX();
        bool result = bibtexExporter->save( &buffer, element, errorLog );
        buffer.close();
        delete bibtexExporter;

        if ( result )
            result = generateOutput( buffer, iodevice );

        return result;
    }

    bool BibTeXFileExporterExternal::generateOutput( QBuffer &input, QIODevice *output )
    {
        bool result = FALSE;

        QString commandLine;
        switch ( m_fileformat )
        {
        case KBibTeX::Settings::formatHTML:
            switch ( m_exporter )
            {
            case KBibTeX::Settings::exporterBibTeX2HTML:
                commandLine = "bibtex2html -s plain -a";
                break;
            case KBibTeX::Settings::exporterBib2XHTML:
                commandLine = "bib2xhtml -s plain -u";
                break;
            case KBibTeX::Settings::exporterBibConv:
                commandLine = "bibconv -informat=bibtex -outformat=html";
                break;
            default:
                return FALSE;
            }
            break;
        default:
            return FALSE;
        }
        QStringList args = QStringList::split( ' ', commandLine );

        writeTo = new QTextStream( output );
        writeTo->setEncoding( QTextStream::UnicodeUTF8 );

        QApplication::setOverrideCursor( Qt::waitCursor );

        process = new QProcess( args );
        connect( process, SIGNAL( processExited() ), this, SLOT( slotProcessExited() ) );
        connect( process, SIGNAL( readyReadStdout() ), this, SLOT( slotReadProcessOutput() ) );
        connect( process, SIGNAL( readyReadStderr() ), this, SLOT( slotReadProcessOutput() ) );
        connect( process, SIGNAL( wroteToStdin() ), this, SLOT( slotWroteToStdin() ) );

        if ( process->start() )
        {
            while ( !process->isRunning() )
            {
                wc->wait( 250 );
                qApp->processEvents();
            }
            qApp->processEvents();
            input.open( IO_ReadOnly );
            QByteArray buf = input.buffer();
            //             qDebug( "buf.size=%i", buf.size() );
            process->writeToStdin( buf );
            input.close();
            qApp->processEvents();
            while ( process->isRunning() )
            {
                wc->wait( 250 );
                qApp->processEvents();
            }

            result = process->normalExit();
        }

        disconnect( process, SIGNAL( wroteToStdin() ), this, SLOT( slotWroteToStdin() ) );
        disconnect( process, SIGNAL( readyReadStdout() ), this, SLOT( slotReadProcessOutput() ) );
        disconnect( process, SIGNAL( readyReadStderr() ), this, SLOT( slotReadProcessOutput() ) );
        disconnect( process, SIGNAL( processExited() ), this, SLOT( slotProcessExited() ) );
        delete( process );
        process = NULL;
        delete writeTo;
        writeTo = NULL;

        QApplication::restoreOverrideCursor();
        return result;
    }

    void BibTeXFileExporterExternal::slotProcessExited()
    {
        wc->wakeAll();
    }

    void BibTeXFileExporterExternal::slotReadProcessOutput()
    {
        if ( writeTo != NULL )
            while ( process->canReadLineStdout() )
            {
                QString line = process->readLineStdout();
                //                 qDebug( "line=%s", line.latin1() );
                ( *writeTo ) << line.latin1() << endl;
            }
        //         while ( process->canReadLineStderr() )
        //         {
        //             QString line = process->readLineStderr();
        //             qDebug( "stderr=%s", line.latin1() );
        //         }
    }

    void BibTeXFileExporterExternal::slotWroteToStdin()
    {
        process->closeStdin();
    }
}

#include "bibtexfileexporterexternal.moc"
